//
//  MyDocViewController.h
//  objcdoc
//
//  Created by zyl910 on 13-6-4.
//  Copyright (c) 2013年 zyl910. All rights reserved.
//

/**
 * @file	MyDocViewController.h
 * @brief	主页面.
 * @author	[zyl910](http://www.cnblogs.com/zyl910/)
 * @version	1.0
 * @date	2013-06-07
 *
 * # update （更新日志）
 *
 * [2013-06-07] <zyl910> v1.0
 *
 * + v1.0版发布.
 *
 */

#import <UIKit/UIKit.h>


#define BUFSIZE	100	//!< 缓冲区大小 （简单宏, 仅Doxygen）.

/**
 *	@brief	最小值 （参数宏, 仅Doxygen）.
 *
 *	@param 	a 	值a.
 *	@param 	b 	值b.
 *
 *	@return	返回两者中的最小值.
 */
#define min(a,b)	( ((a)<(b)) ? (a) : (b) )


/// Objective-C 文档工具枚举 （枚举, 仅Doxygen）.
typedef enum _ObjCDocToolEnum{
	ObjCDocToolEnumAppleDoc = 1,	//!< AppleDoc. http://appledoc.gentlebytes.com/ .
	ObjCDocToolEnumDoxygen,	//!< Doxygen. http://www.stack.nl/~dimitri/doxygen/ .
}ObjCDocToolEnum;


/** 整数矩形 （结构体, 仅Doxygen）.
 *
 * 结构体的详细描述.
 */
typedef struct _RectInt {
	int x;	//!< 横坐标.
	int y;	//!< 纵坐标.
	int width;	//!< 宽度.
	int height;	//!< 高度.
}RectInt, *PRectInt;	//!< 整数矩形的指针.
typedef const RectInt* PCRectInt;	//!< 整数矩形的常量指针.


/// 浮点数的字节（联合体, 仅Doxygen）.
typedef union _FloatByte {
	float f;	//!< 单精度浮点数.
	unsigned char bytes[4];	//!< 4个字节.
} FloatByte;


/**
 *	@brief	动作回调函数.
 *
 *	@param 	sender 	发送者.
 *	@param 	userdata 	自定义数据.
 */
typedef void (*ActionCallback)(void* sender, void* userdata);


/**
 *	@brief	动作块函数.
 *
 *	@param 	sender 	发送者.
 *	@param 	userdata 	自定义数据.
 */
typedef void (^ActionHandler)(id sender, id userdata);


/// 文档A.
@interface DocA : NSObject
@end;

/** 文档B.
 *
 * 文档B的详细描述.
 */
@interface DocB : NSObject
@end;


/// Objective-C 文档委托.
@protocol MyDocDelegate <NSObject>

@end

/** 主页面.
 *
 * 主页面的详细描述. 
 *
 * 引用短代码, 如 `someMethodByStr:` .
 *
 * 示例代码:
 *
 * 	int sum=0;
 * 	for(int i=1; i<=10; ++i) {
 * 		sum += i;
 * 	}
 *
 * 无序列表:
 *
 * - abc
 * - xyz
 * - rgb
 *
 * 有序列表:
 *
 * 1. first.
 * 2. second.
 * 3. third.
 *
 * 多级列表:
 *
 * - xyz
 *	- x
 *	- y
 *	- z
 * - rgb
 *	- red
 *		1. first.
 *			1. alpha.
 *			2. beta.
 *		2. second.
 *		3. third.
 *	- green
 *	- blue
 *
 * 以下是仅doxygen可见内容. 因为appledoc不支持类型注释中的\@see标签(但支持在属性、方法中使用)，会导致后续内容丢弃.
 *
 * @see str
 * @see someMethodByStr:
 * @see MyDocAppDelegate
 * @see [MyDocAppDelegate window]	// 仅 appledoc. doxygen仅能识别出左边的类名.
 * @see [MyDocAppDelegate someMethodByFloat:]	// 仅 appledoc. doxygen仅能识别出左边的类名.
 * @see MyDocAppDelegate.window	// 仅 doxygen. appledoc仅能识别出左边的类名.
 * @see MyDocAppDelegate.someMethodByFloat:	// 仅 doxygen. appledoc仅能识别出左边的类名.
 *
 * end.
 *
 */
@interface MyDocViewController : UIViewController {
	@private
	int _privateInt;	//!< 私有成员变量 (仅Doxygen具有EXTRACT_PRIVATE标识时, 会被归类为“Private 属性”).
	
	@protected
	int _protectedInt;	//!< protected成员变量 (仅Doxygen, 会被归类为“Protected 属性”).
	id<MyDocDelegate> _delegate;	//!< 委托变量.
	
	@package
	int _packageInt;	//!< 包内成员变量 (仅Doxygen, 会被归类为“Protected 属性”).
	
	@public
	int _publicInt;	//!< 公开成员变量 (仅Doxygen, 会被归类为“Public 属性”).
}

#pragma mark - property

/// 数值属性.
@property (nonatomic,assign) NSInteger num;

/** 字符串属性.
 *
 * 属性的详细描述.
 */
@property (nonatomic,strong) NSString* str;


// 测试行尾注释.
@property (nonatomic,strong) NSString* strend1;	/**< 行尾注释1. appledoc不支持会变为下一项的注释, doxygen 支持, 根据英文句号自动切分详细描述与详细描述. */
@property (nonatomic,strong) NSString* strend2;	/*!< 行尾注释2. appledoc不支持会变为下一项的注释, doxygen 支持, 会全部当作详细描述, 而缺少详细描述. */
@property (nonatomic,strong) NSString* strend3;	///< 行尾注释3. appledoc不支持会变为下一项的注释, doxygen 支持.
@property (nonatomic,strong) NSString* strend4;	//!< 行尾注释4. appledoc不支持会会忽略, doxygen 支持.
@property (nonatomic,assign) int dummy;


/// 枚举的属性.
@property (nonatomic, assign) ObjCDocToolEnum docTool;

/// 结构体的属性.
@property (nonatomic, assign) RectInt rectInt;

/// 结构体常量指针的属性.
@property (nonatomic, assign) PCRectInt prectInt;

/// 联合体的属性.
@property (nonatomic, assign) FloatByte floatByte;

/// 委托.
@property (nonatomic, strong) id<MyDocDelegate> delegate;


/** 链接测试.
 *
 * 链接:
 *
 * - http://appledoc.gentlebytes.com/ : 直接写url链接.
 * - [Doxygen](http://www.stack.nl/~dimitri/doxygen/) : 为链接提供文本 .
 * - MyDocDelegate : 接口.
 * - DocA : 类.
 * - [文档B](DocB) : 类的链接文本.（仅appledoc）
 * - [文档B](@ref DocB) : 为\@ref链接提供文本 （仅doxygen. appledoc会把\@ref当作文本而生成错误的链接）.
 * - @ref DocB : \@ref链接 （仅doxygen. appledoc会把\@ref当作文本）.
 *
 * @see str
 * @see someMethodByStr:
 * @see MyDocAppDelegate
 * @see [MyDocAppDelegate window]	// 仅 appledoc. doxygen仅能识别出左边的类名.
 * @see [MyDocAppDelegate someMethodByFloat:]	// 仅 appledoc. doxygen仅能识别出左边的类名.
 * @see MyDocAppDelegate.window	// 仅 doxygen. appledoc仅能识别出左边的类名.
 * @see MyDocAppDelegate.someMethodByFloat:	// 仅 doxygen. appledoc仅能识别出左边的类名.
 */
@property (nonatomic,strong) NSString* alink;

/** 本地链接测试1 （仅appledoc）.
 *
 * 链接:
 *
 * - str : 自身属性.
 * - someMethodByStr: : 自身方法.
 * - MyDocViewController : 自身类.	// appledoc无法识别.
 * - MyDocAppDelegate : 外部类.
 */
@property (nonatomic,strong) NSString* alinklocal1;

/** 本地链接测试2 （仅appledoc）.
 *
 * 只要出现了 \[ 类 成员 \] 形式的链接, 本地链接便会实效.
 *
 * 链接:
 *
 * - str : 自身属性.
 * - someMethodByStr: : 自身方法.
 * - MyDocViewController : 自身类.	// appledoc无法识别.
 * - MyDocAppDelegate : 外部类.
 * - [MyDocViewController str] : 自身属性.
 */
@property (nonatomic,strong) NSString* alinklocal2;

/** 本地链接测试3 （仅appledoc）.
 *
 * 只要出现了 \[ 类 成员 \] 形式的链接, 本地链接便会实效.
 *
 * 链接:
 *
 * - str : 自身属性.
 * - someMethodByStr: : 自身方法.
 * - MyDocViewController : 自身类.	// appledoc无法识别.
 * - MyDocAppDelegate : 外部类.
 * - [MyDocAppDelegate window] : 外部类属性.
 * - [MyDocAppDelegate someMethodByFloat:] : 外部类方法.
 */
@property (nonatomic,strong) NSString* alinklocal3;


#pragma mark - method

/// 简单方法.
- (void)someMethod;

/**
 *	@brief	带整数参数的方法.
 *
 *	@param 	value	值.
 *
 *	@return	返回value.
 */
- (int)someMethodByInt:(int)value;

/**
 *	@brief	带字符串参数的方法.
 *
 *	@param 	value	值.
 *
 *	@return	返回value.
 *
 *	@exception NSException 可能抛出的异常.
 *
 *	@see someMethod
 *	@see someMethodByInt:
 *	@warning 警告: appledoc中显示为蓝色背景, Doxygen中显示为红色竖条.
 *	@bug 缺陷: appledoc中显示为黄色背景, Doxygen中显示为绿色竖条.
 */
- (NSString*)someMethodByStr:(NSString*)value;


/**
 *	@brief	取得静态变量值的类方法.
 *
 *	@return	返回_classInt静态变量的值.
 */
+ (int)classInt;

@end


/// 主页面的动作相关操作.
@interface MyDocViewController (Action)

/**
 *	@brief	调用动作回调函数
 *
 *	@param 	sender 	发送者.
 *	@param 	userdata 	自定义数据.
 *	@param 	handler 	处理函数.
 *
 *	@see callActionHandler:userdata:handler:
 */
- (void)callActionCallback:(void*)sender userdata:(void*)userdata handler:(ActionCallback)handler;

/**
 *	@brief	调用动作块函数.
 *
 *	@param 	sender 	发送者.
 *	@param 	userdata 	自定义数据.
 *	@param 	handler 	处理函数.
 *
 *	@see callActionCallback:userdata:handler:
 */
- (void)callActionHandler:(id)sender userdata:(id)userdata handler:(ActionHandler)handler;


@end
